/* 
* I2C.cpp
*
* Created: 2014/06/21 14:04:25
* Author: SIMON
*/

#ifndef _I2C
#define _I2C

#include "I2C.h"
#include <util/delay.h>

I2C::I2C()
{
	SetTWEN(1);
	SetTWEA(1);
	MaxLoopTime = 10000;
	ReadOrWrite = 0;
}

unsigned char I2C::GetStatus(){
	unsigned char val;
	val = (TWSR & 0xF8);
	return val;
}

void I2C::SetWriteAddress(unsigned char _WriteAddress){
	Address = _WriteAddress;
}

void I2C::SetOwnAddress(unsigned char _OwnAddress, unsigned char EnGeneralCall){
	TWAR = ((_OwnAddress << 1) | EnGeneralCall);
}

void I2C::SetBR(unsigned char _BR,unsigned char _PS){
	TWBR = _BR;
	TWSR &= ~0x03;
	if (_PS == 1){
		TWSR |= 0x00;
	}else if (_PS == 4){
		TWSR |= 0x01;
	}else if (_PS == 16){
		TWSR |= 0x02;
	}else{
		TWSR |= 0x00;
	}
}

void I2C::SetRW(unsigned char _RW){
	ReadOrWrite = _RW;
}

void I2C::SetTWEA(unsigned char val){
	if (val == 1){
		TWCR |= 0x40;
	}else{
		TWCR &= ~0x40;
	}
}

void I2C::SetTWEN(unsigned char val){
	if (val == 1){
		TWCR |= 0x04;
		}else{
		TWCR &= ~0x04;
	}
}

void I2C::ClearIntFlag(){
	TWCR |= 0x80;
}

void I2C::CheckAndClearIntFlag(){
	if (CheckIntFlag() == 1){
		ClearIntFlag();
	}
}

void I2C::SetMaxLoopTime(unsigned char val){
	MaxLoopTime = val;
}

unsigned char I2C::LoopUntilFlagSet(){
	unsigned char i=0;
	while((TWCR & 0x80) != 0x80){
		_delay_us(1);
		i++;
		if (i >= MaxLoopTime){
			return 0;
		}
	}
	return 1;
}

void I2C::SetTWSTA(unsigned char val){
	if (val == 1){
		TWCR |= 0x20;
	}else{
		TWCR &= ~0x20;
	}
}

void I2C::SetTWSTO(unsigned char val){
	if (val == 1){
		TWCR |= 0x10;
	}else{
		TWCR &= ~0x10;
	}
}

void I2C::SetTWDRData(unsigned char _Data){
	TWDR = _Data;
}

void I2C::WriteToSlave(unsigned char _Data){
	unsigned char i=0;
	SetTWSTA(1);
	LoopUntilFlagSet();
	while(GetStatus() != 0x08){
		_delay_us(1);
		i++;
		if (i >= MaxLoopTime){
			i=0;
			return;
		}
	}
	i=0;
	TWDR = Address << 1;
	SetTWSTA(0);
	ClearIntFlag();
	LoopUntilFlagSet();
	while (GetStatus() != 0x18){
		_delay_us(1);
		i++;
		if (i >= MaxLoopTime){
			i=0;
			return;
		}
	}
	i=0;
	TWDR = _Data;
	ClearIntFlag();
	LoopUntilFlagSet();
	while(GetStatus() != 0x28){
		_delay_us(1);
		i++;
		if (i >= MaxLoopTime){
			i=0;
			return;
		}
	}
	i=0;
	SetTWSTO(1);
}

unsigned char I2C::ReadFromMaster(){
	unsigned char val=0;
	unsigned char i=0;
	LoopUntilFlagSet();
	while (GetStatus() != 0x60){
		_delay_us(1);
		i++;
		if (i >= MaxLoopTime){
			return 0;
		}
	}
	i=0;
	ClearIntFlag();
	LoopUntilFlagSet();
	while(GetStatus() != 0x80){
		_delay_us(1);
		i++;
		if (i >= MaxLoopTime){
			i=0;
			return 0;
		}
	}
	i=0;
	val = TWDR;
	ClearIntFlag();
	LoopUntilFlagSet();
	while(GetStatus() != 0xA0){
		_delay_us(1);
		i++;
		if (i >= MaxLoopTime){
			i=0;
			return 0;
		}
	}
	i=0;
	ClearIntFlag();
	return val;
}


unsigned char I2C::CheckIntFlag(){
	unsigned char val;
	if ((TWCR & 0x80) == 0x80){
		val = 1;
	}else{
		val = 0;
	}
	return val;
}

// default destructor
I2C::~I2C()
{
} //~I2C

#endif